/*
 *                            COPYRIGHT
 *
 *  PCB, interactive printed circuit board design
 *  Copyright (C) 1994,1995 Thomas Nau
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Contact addresses for paper mail and Email:
 *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
 *  Thomas.Nau@rz.uni-ulm.de
 *
 */

static	char	*rcsid = "$Header: /sda7/users/nau/src/pcb/pcb-1.4.0/RCS/printpanner.c,v 1.4 1996/01/13 16:35:39 nau Exp $";

/* print dialog panner routines
 * only available for X11R5 and later
 * all routines are defined 'empty' for X11R4
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <X11/Xlib.h>

#include "global.h"

#include "data.h"
#include "menu.h"
#include "misc.h"
#include "printpanner.h"

/* ---------------------------------------------------------------------------
 * some local identifiers
 */
static	MediaTypePtr	MediaSelection = NULL;	/* selected media */
static	MediaType		Media[] = {				/* first one is user defined */
	{ "user defined", 0, 0, 0, 0 },
	{ "a3", 11690, 16530, 500, 500 },
	{ "a4", 8260, 11690, 500, 500 },
	{ "a5", 5830, 8260, 500, 500 },
	{ "letter", 8500, 11000, 500, 500 },
	{ "tabloit", 11000, 17000, 500, 500 },
	{ "ledger", 17000, 11000, 500, 500 },
	{ "legal", 8500, 14000, 500, 500 },
	{ "executive", 7500, 10000, 500, 500 }};
static	PopupEntryType	MediaMenuEntries[ENTRIES(Media)+1];
static	PopupMenuType	MediaMenu =
	{ "media", NULL, MediaMenuEntries, NULL, NULL, NULL };
static	MenuButtonType	MediaMenuButton =
	{ "media", "Media", &MediaMenu, NULL };

/* ---------------------------------------------------------------------------
 * a local prototype for both cases
 */
static	void	InitDefaultMedia(void);

/***************************************************************************** 
 * routines for X11R5 an newer
 *****************************************************************************/
#if XlibSpecificationRelease >= 5

#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Panner.h>

/* ---------------------------------------------------------------------------
 * some local prototypes
 */
static	void	CB_Media(Widget, XtPointer, XtPointer);
static	Widget	PrintPannerCreateMediaMenu(Widget, Widget, Widget);

/* ---------------------------------------------------------------------------
 * some local identifiers
 */
static	Widget			Panner,			/* the panner widget */
						SizeField;		/* label to display the media size */
static	Dimension		Width, Height;	/* layout size */

/* ---------------------------------------------------------------------------
 * returns pointer to current media selection
 */
MediaTypePtr PrintPannerGetMedia(void)
{
	return(MediaSelection);
}

/* ---------------------------------------------------------------------------
 * updates the panner widget
 */
void PrintPannerUpdate(float Scale, Boolean RotateFlag)
{
	Dimension	width, height;

		/* scale layout size */
	width = (Dimension) ((float) Width *Scale);
	height = (Dimension) ((float) Height *Scale);

		/* update slider */
	XtVaSetValues(Panner,
		XtNsliderWidth, RotateFlag ? height : width,
		XtNsliderHeight, RotateFlag ? width : height,
		NULL);
}

/* ---------------------------------------------------------------------------
 * callback for media size selection (menu)
 */
static void CB_Media(Widget W, XtPointer ClientData, XtPointer CallData)
{
	MediaTypePtr	media = (MediaTypePtr) ClientData;
	char			size[80];
	Dimension		mediawidth,
					mediaheight;

		/* print visible size of media */
	mediawidth = media->Width -2*media->MarginX;
	mediaheight = media->Height -2*media->MarginY;
	sprintf(size, "media size:     %4.2f\" x %4.2f\"\n"
				  "without margin: %4.2f\" x %4.2f\"\n",
		(float) media->Width /1000.0,
		(float) media->Height /1000.0,
		(float) mediawidth /1000.0,
		(float) mediaheight /1000.0);

		/* update widgets */
	XtVaSetValues(MediaMenuButton.W, XtNlabel, media->Name, NULL);
	XtVaSetValues(SizeField, XtNlabel, size, NULL);
	XtVaSetValues(Panner,
		XtNcanvasHeight, mediaheight,
		XtNcanvasWidth, mediawidth,
		NULL);
}

/* ---------------------------------------------------------------------------
 * creates 'media size' menu
 */
static Widget PrintPannerCreateMediaMenu(Widget Parent, Widget Top, Widget Left)
{
	int		i;

		/* copy media description to menuentry structure */
	for (i = 0; i < ENTRIES(Media); i++)
	{
		MediaMenuEntries[i].Name = Media[i].Name;
		MediaMenuEntries[i].Label = Media[i].Name;
		MediaMenuEntries[i].Callback = CB_Media;
		MediaMenuEntries[i].ClientData = (char *) &Media[i];
		MediaMenuEntries[i].W = NULL;
	}
	return(InitMenuButton(Parent, &MediaMenuButton, Top, Left));
}

/* ---------------------------------------------------------------------------
 * creates a print command dialog
 */
Widget PrintPannerCreate(Widget Parent, Widget Top, Widget Left)
{
	Widget		label,
				menu;
	BoxTypePtr	box;

		/* init media size if called the first time */
	if (!MediaSelection)
		InitDefaultMedia();

	label = XtVaCreateManagedWidget("comment", labelWidgetClass,
		Parent,
		LAYOUT_TOP,
		XtNfromVert, Top,
		XtNfromHoriz, Left,
		XtNlabel, "select media and offsets:",
		NULL);
	menu = PrintPannerCreateMediaMenu(Parent, label, Left);
	SizeField = XtVaCreateManagedWidget("sizeField", labelWidgetClass,
		Parent,
		LAYOUT_TOP,
		XtNfromVert, menu,
		XtNfromHoriz, Left,
		XtNlabel, "",
		NULL);
	Panner = XtVaCreateManagedWidget("panner", pannerWidgetClass,
		Parent,
		LAYOUT_TOP,
		XtNdefaultScale, 1,
		XtNfromVert, Top,
		XtNfromHoriz, SizeField,
		XtNrubberBand, True,
		XtNresizable, True,
		NULL);

		/* get layout size and initialize size label;
		 * layout has already be checked to be non-empty
		 */
	box = GetDataBoundingBox(PCB->Data);
	Width = box->X2 -box->X1;
	Height = box->Y2 -box->Y1;
	CB_Media(NULL, (XtPointer) MediaSelection, NULL);
	return(Panner);
}

/* ----------------------------------------------------------------------
 * return the selected offset and height of slider
 */
void PrintPannerGetOffset(Position *OffsetX, Position *OffsetY)
{
	XtVaGetValues(Panner,
		XtNsliderX, OffsetX,
		XtNsliderY, OffsetY,
		NULL);
}

/* ----------------------------------------------------------------------
 * set the panner offset
 */
void PrintPannerSetSliderPosition(Position OffsetX, Position OffsetY)
{
	XtVaSetValues(Panner,
		XtNsliderX, OffsetX,
		XtNsliderY, OffsetY,
		NULL);
}

#else

/***************************************************************************** 
 * routines for X11R4 an older; just some empty routines
 *****************************************************************************/
void PrintPannerUpdate(float Scale, Boolean RotateFlag)
{
}

Widget PrintPannerCreate(Widget Parent, Widget Top, Widget Left)
{
	if (!MediaSelection)
		InitDefaultMedia();
	return(Top);
}

void PrintPannerSetSliderPosition(Position OffsetX, Position OffsetY)
{
}

void PrintPannerGetOffset(Position *OffsetX, Position *OffsetY)
{
		/* always pass zero as offset */
	*OffsetX = 0;
	*OffsetY = 0;
}

/* ---------------------------------------------------------------------------
 * returns pointer to current media selection
 */
MediaTypePtr PrintPannerGetMedia(void)
{
	return(NULL);
}

#endif

/***************************************************************************** 
 * routines common to all X releases
 *****************************************************************************/

/* ---------------------------------------------------------------------------
 * initializes the user defined mediasize and sets it as default
 * only called once
 */
static void InitDefaultMedia(void)
{
	int				i,
					x, y;
	unsigned int	width, height;
	char			*p1, *p2;

		/* check if the passed string is equal to a known media type
		 * first entry is user defined, last one holds a zero pointer only
		 */
	for (i = 1; i < ENTRIES(Media)-1; i++)
	{
		p1 = Media[i].Name;
		p2 = Settings.Media;

			/* ignore case */
		for (; *p1 && *p2 && (tolower(*p1) == tolower(*p2)); p1++, p2++);

			/* found a match */
		if (*p1 == *p2)
			break;
	}

	if (MediaSelection != &Media[ENTRIES(Media)-1])
	{
			/* used the found entry */
		MediaSelection = &Media[i];
		width = Media[i].Width;
		height = Media[i].Height;
		x = Media[i].MarginX;
		y = Media[i].MarginY;
	}
	else
	{
			/* no match, try to evaluate it as geometry string;
			 * use first entry (user defined)
			 */
		MediaSelection = Media;
		i = XParseGeometry(Settings.Media, &x, &y, &width, &height);

			/* check the syntax */
		if ((i & WidthValue) || (i & HeightValue) ||
			(i & XValue) || (i & YValue))
		{
				/* failed; use default setting (a4), no margin */
			width = 8260;
			height = 11690;
			x = y = 0;
		}
	}

		/* set size as user default; MediaSelection has already been set != NULL
		 * to make sure that routine wont be called again
		 */
	Media[0].Width = width;
	Media[0].Height = height;
	Media[0].MarginX = x;
	Media[0].MarginY = y;
}

